# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from .. import helper
from ..constants.ansi import RESET_ALL
from ..model.background.bright_color import BgBrightColor
from ..model.background.color import BgColor
from ..model.background.hex import BgColorHex
from ..model.background.hsl import BgColorHSL
from ..model.background.oklch import BgColorOKLCH
from ..model.background.rgb import BgColorRGB
from ..model.background.vga import BgColorVGA
from ..model.effect import Effect
from ..model.foreground.bright_color import BrightColor
from ..model.foreground.color import Color
from ..model.foreground.hex import ColorHex
from ..model.foreground.hsl import ColorHSL
from ..model.foreground.oklch import ColorOKLCH
from ..model.foreground.rgb import ColorRGB
from ..model.foreground.vga import ColorVGA


def print_color(text: str,
                color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | ColorOKLCH | str | None = None,
                bg_color: BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | BgColorOKLCH | str | None = None,
                effect: Effect | str | None = None
                ) -> None:
    """Prints full line of text with various options for text and background color, styling and effects."""

    color_str = helper.print.normalize_input(color)
    bg_color_str = helper.print.normalize_input(bg_color)
    effect_str = helper.print.normalize_input(effect)

    print(f"{color_str}{bg_color_str}{effect_str}{text}{RESET_ALL}")



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from abc import ABC, abstractmethod

from ... import helper
from ...constants.ansi import RESET_ALL
from ...helper.error import (message_for_hue_value_error,
                             message_for_oklch_chroma_value_error,
                             message_for_percentage_as_decimal_value_error)
from ...helper.validate import (is_valid_hue_value,
                                is_valid_oklch_chroma_value,
                                is_valid_percentage_as_decimal)
from ..foreground.rgb import ColorRGB
from .rgb import RGB_ABC


class OKLCH_ABC(ABC):
    """Abstract base class for custom OKLCH color instances."""

    OFF = RESET_ALL

    __slots__ = ["chroma", "hue", "lightness", "_rgb", "_ansi_code"]

    def __init__(self, lightness: float, chroma: float, hue: float):
        if not is_valid_percentage_as_decimal(lightness):
            raise ValueError(message_for_percentage_as_decimal_value_error("lightness", lightness))
        if not is_valid_oklch_chroma_value(chroma):
            raise ValueError(message_for_oklch_chroma_value_error(chroma))
        if not is_valid_hue_value(hue):
            raise ValueError(message_for_hue_value_error(hue))
        self.hue: float = hue
        self.chroma: float = chroma
        self.lightness: float = lightness

        self._rgb: RGB_ABC = self.convert_oklch_to_srgb()
        self._ansi_code: str = self.generate_ansi_code()

    def __str__(self) -> str:
        return self._ansi_code

    def __repr__(self) -> str:
        return f"L: {self.lightness}, C: {self.chroma}, H: {self.hue}"

    def convert_oklch_to_srgb(self) -> RGB_ABC:
        """Method to convert OKLCH to RGB color."""

        red, green, blue = helper.convert.oklch_to_srgb(self.lightness, self.chroma, self.hue)
        return ColorRGB(red, green, blue)

    @abstractmethod
    def generate_ansi_code(self) -> str:
        """Method to generate ANSI RGB color sequence."""

        raise NotImplementedError  # pragma: no cover



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.background.oklch import BgColorOKLCH
from ...print.general import print_color


def bg_oklch(text: str, lightness: float, chroma: float, hue: float) -> None:
    """Prints full line of text on custom OKLCH-colored background.

    Args:
        text (str): Text to be printed on colored background.
        lightness (float): Lightness value between `0.0` and `1.0` where `0.0` is black and `1.0` is white.
        chroma (float): Chroma value between `0.0` and `0.4`.
        hue (float): Number between `0` and `360` degrees.
    """

    bg_color_oklch = BgColorOKLCH(lightness, chroma, hue)
    print_color(text, bg_color=bg_color_oklch)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def bg_oklch(self, text: str, lightness: float, chroma: float, hue: float) -> None:
        """Prints full line of text on custom OKLCH-colored background.

        Args:
            text (str): Text to be printed on colored background.
            lightness (float): Lightness value between `0.0` and `1.0` where `0.0` is black and `1.0` is white.
            chroma (float): Chroma value between `0.0` and `0.4`.
            hue (float): Number between `0` and `360` degrees.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_oklch

            bg_oklch("I want this background in green OKLCH colors", 0.54, 0.15, 141)
            ```

            How it appears in the terminal:

            <pre><code>% <span class="text-contrast" style="background-color: oklch(0.54 0.15 141)">I want this background in green OKLCH colors</span></code></pre>
        """
